﻿using System;
using System.Threading;
using MBN;
using MBN.Enums;
using MBN.Extensions;
using MBN.Modules;
using Microsoft.SPOT;

namespace Examples
{

    // Set the DateTime for first power up only
    // WARNING - Do not attempt to adjust the System Time using Microsoft.SPOT.Hardware.Utility.SetLocalTime()
    // or it will lock up your MBN Board and it will have to be re-flashed. Trust me on this.
    // A Bug Report has been submitted to Justin

    public class Program
    {
        private static RTCClick _clock1;

        private static readonly DateTime NDate = new DateTime(2014, 5, 29, 23, 59, 50);
        private static readonly DateTime LDate = new DateTime(2016, 2, 28, 23, 59, 50);
        private static readonly DateTime YDate = new DateTime(2014, 12, 31, 23, 59, 50);

        static int _iterations = 0;

        public static void Main()
        {
            _clock1 = new RTCClick(Hardware.SocketThree, ClockRatesI2C.Clock400KHz, 100);

            _clock1.AlarmTriggered += _clock1_AlarmTriggered;

            Debug.Print("**********RTC Click Demo**********");
            Debug.Print("\nDriver Version - " + _clock1.DriverVersion);
            Debug.Print("Is DateTime set? " + _clock1.IsDateTimeSet);

            Debug.Print("Is Leap Year" + LDate.IsLeapYear());

            DateRolloverDemo();
            LeapYearDemo();
            YearRolloverDemo();
            ClockStopperDemo();
            UserRamDemo();

            // Select one of the three Alarm Tests below:
            //DailyAlarmTest();
            //WeekDayAlarmTest();
            DatedAlarmTest();

            while (true)
            {
                var dt = _clock1.GetDateTime();
                Debug.Print("RTC DateTime - " + dt + "\n");

                Thread.Sleep(1 * 1000);
            }
        }

        private static void DoPoop()
        {
            var thisDate = new DateTime(2016, 2, 28, 23, 59, 50);
            Debug.Print("Is Leap Year" + LDate.IsLeapYear());
        }

        private static void DatedAlarmTest()
        {
            Debug.Print("\n" + "**********Doing Dated Alarm Test**********");

            _clock1.AlarmAutoEnable = true;
            _clock1.AlarmAutoReset = true;

            _clock1.SetClock(new DateTime(2014, 8, 19, 23, 59, 55));

            _clock1.ClearAlarmSettings();

            _clock1.SetDatedAlarm(2014, 8, 20, 0, 0, 5);

            Debug.Print("\nAlarm Settings");

            RTCClick.AlarmData aData = _clock1.GetAlarmSettings();
            aData = _clock1.GetAlarmSettings();
            Debug.Print("Hundreds - " + aData.Time.HundredthsSecond);
            Debug.Print("Seconds - " + aData.Time.Seconds);
            Debug.Print("Minutes - " + aData.Time.Minutes);
            Debug.Print("Hours - " + aData.Time.Hours);
            Debug.Print("Day - " + aData.Day);
            Debug.Print("Day of Week - " + (aData.DayOfWeek == RTCClick.WeekDays.Sunday ? "Sunday" : aData.DayOfWeek == RTCClick.WeekDays.Monday ? "Monday" : aData.DayOfWeek == RTCClick.WeekDays.Tuesday ? "Tuesday" : aData.DayOfWeek == RTCClick.WeekDays.Wednesday ? "Wednesday" : aData.DayOfWeek == RTCClick.WeekDays.Thursday ? "Thursday" : aData.DayOfWeek == RTCClick.WeekDays.Friday ? "Friday" : aData.DayOfWeek == RTCClick.WeekDays.Saturday ? "Saturday " : aData.DayOfWeek == RTCClick.WeekDays.Weekdays ? "Weekdays" : aData.DayOfWeek == RTCClick.WeekDays.Weekends ? "Weekends" : aData.DayOfWeek == RTCClick.WeekDays.AllDays ? "All Days" : aData.DayOfWeek == RTCClick.WeekDays.None ? "None" : "Multiple Days"));
            Debug.Print("Month - " + aData.Month);
            Debug.Print("Alarm Type - " + (aData.AlarmType == RTCClick.AlarmBehavior.DailyAlarm ? "Daily Alarm" : aData.AlarmType == RTCClick.AlarmBehavior.WeekDayAlarm ? "Weekday Alarm" : aData.AlarmType == RTCClick.AlarmBehavior.DatedAlarm ? "Dated Alarm" : "No alarm set.") + "\n");

            Debug.Print("RTC Initialized? " + _clock1.IsDateTimeSet);
            Debug.Print("Alarm Set? " + _clock1.IsAlarmTimeSet + "\n");
        }

        private static void DailyAlarmTest()
        {
            Debug.Print("\n" + "**********Doing Daily Alarm Test**********");

            _clock1.AlarmAutoEnable = true;
            _clock1.AlarmAutoReset = true;

            _clock1.SetClock(new DateTime(2014, 8, 19, 23, 59, 55));

            _clock1.ClearAlarmSettings();

            _clock1.SetDailyAlarm(0, 0, 5);

            Debug.Print("\nAlarm Settings");

            RTCClick.AlarmData aData = _clock1.GetAlarmSettings();
            aData = _clock1.GetAlarmSettings();
            Debug.Print("Hundreds - " + aData.Time.HundredthsSecond);
            Debug.Print("Seconds - " + aData.Time.Seconds);
            Debug.Print("Minutes - " + aData.Time.Minutes);
            Debug.Print("Hours - " + aData.Time.Hours);
            Debug.Print("Day - " + aData.Day);
            Debug.Print("Day of Week - " + (aData.DayOfWeek == RTCClick.WeekDays.Sunday ? "Sunday" : aData.DayOfWeek == RTCClick.WeekDays.Monday ? "Monday" : aData.DayOfWeek == RTCClick.WeekDays.Tuesday ? "Tuesday" : aData.DayOfWeek == RTCClick.WeekDays.Wednesday ? "Wednesday" : aData.DayOfWeek == RTCClick.WeekDays.Thursday ? "Thursday" : aData.DayOfWeek == RTCClick.WeekDays.Friday ? "Friday" : aData.DayOfWeek == RTCClick.WeekDays.Saturday ? "Saturday " : aData.DayOfWeek == RTCClick.WeekDays.Weekdays ? "Weekdays" : aData.DayOfWeek == RTCClick.WeekDays.Weekends ? "Weekends" : aData.DayOfWeek == RTCClick.WeekDays.AllDays ? "All Days" : aData.DayOfWeek == RTCClick.WeekDays.None ? "None" : "Multiple Days"));
            Debug.Print("Month - " + aData.Month);
            Debug.Print("Alarm Type - " + (aData.AlarmType == RTCClick.AlarmBehavior.DailyAlarm ? "Daily Alarm" : aData.AlarmType == RTCClick.AlarmBehavior.WeekDayAlarm ? "Weekday Alarm" : aData.AlarmType == RTCClick.AlarmBehavior.DatedAlarm ? "Dated Alarm" : "No alarm set.") + "\n");

            Debug.Print("RTC Initialized? " + _clock1.IsDateTimeSet);
            Debug.Print("Alarm Set? " + _clock1.IsAlarmTimeSet + "\n");
        }

        private static void WeekDayAlarmTest()
        {
            Debug.Print("\n" + "**********Doing Weekday Alarm Test**********");

            _clock1.AlarmAutoEnable = true;
            _clock1.AlarmAutoReset = true;

            _clock1.SetClock(new DateTime(2014, 8, 19, 23, 59, 55));

            _clock1.ClearAlarmSettings();

            RTCClick.WeekDays weekDays = RTCClick.WeekDays.AllDays;
            _clock1.SetWeekdayAlarm(weekDays, 00, 00, 05);

            Debug.Print("\nAlarm Settings");

            RTCClick.AlarmData aData = _clock1.GetAlarmSettings();
            aData = _clock1.GetAlarmSettings();
            Debug.Print("Hundreds - " + aData.Time.HundredthsSecond);
            Debug.Print("Seconds - " + aData.Time.Seconds);
            Debug.Print("Minutes - " + aData.Time.Minutes);
            Debug.Print("Hours - " + aData.Time.Hours);
            Debug.Print("Day - " + aData.Day);
            Debug.Print("Day of Week - " + (aData.DayOfWeek == RTCClick.WeekDays.Sunday ? "Sunday" : aData.DayOfWeek == RTCClick.WeekDays.Monday ? "Monday" : aData.DayOfWeek == RTCClick.WeekDays.Tuesday ? "Tuesday" : aData.DayOfWeek == RTCClick.WeekDays.Wednesday ? "Wednesday" : aData.DayOfWeek == RTCClick.WeekDays.Thursday ? "Thursday" : aData.DayOfWeek == RTCClick.WeekDays.Friday ? "Friday" : aData.DayOfWeek == RTCClick.WeekDays.Saturday ? "Saturday " : aData.DayOfWeek == RTCClick.WeekDays.Weekdays ? "Weekdays" : aData.DayOfWeek == RTCClick.WeekDays.Weekends ? "Weekends" : aData.DayOfWeek == RTCClick.WeekDays.AllDays ? "All Days" : aData.DayOfWeek == RTCClick.WeekDays.None ? "None" : "Multiple Days"));
            Debug.Print("Month - " + aData.Month);
            Debug.Print("Alarm Type - " + (aData.AlarmType == RTCClick.AlarmBehavior.DailyAlarm ? "Daily Alarm" : aData.AlarmType == RTCClick.AlarmBehavior.WeekDayAlarm ? "Weekday Alarm" : aData.AlarmType == RTCClick.AlarmBehavior.DatedAlarm ? "Dated Alarm" : "No alarm set.") + "\n");

            Debug.Print("RTC Initialized? " + _clock1.IsDateTimeSet);
            Debug.Print("Alarm Set? " + _clock1.IsAlarmTimeSet + "\n");
        }

        private static void YearRolloverDemo()
        {
            Debug.Print("\n" + "**********Doing Year roll over test**********");

            _clock1.SetClock(YDate);

            while (true)
            {
                if (_iterations > 20)
                {
                    _iterations = 0;
                    break;
                }
                var dt1 = _clock1.GetDateTime();

                Debug.Print("RTC DateTime - " + dt1);

                Thread.Sleep(1000);
                _iterations += 1;
            }
        }

        private static void LeapYearDemo()
        {
            Debug.Print("\n" + "**********Doing Leap Year test**********");

            _clock1.SetClock(LDate);

            while (true)
            {
                if (_iterations > 20)
                {
                    _iterations = 0;
                    break;
                }
                var dt1 = _clock1.GetDateTime();

                Debug.Print("RTC DateTime - " + dt1);

                Thread.Sleep(1000);
                _iterations += 1;
            }
        }

        private static void DateRolloverDemo()
        {
            _clock1.SetClock(NDate);

            Debug.Print("**********Doing date roll over test**********");

            while (true)
            {
                if (_iterations > 20)
                {
                    _iterations = 0;
                    break;
                }
                var dt1 = _clock1.GetDateTime();

                Debug.Print("RTC DateTime - " + dt1);

                Thread.Sleep(1000);
                _iterations += 1;
            }
        }

        private static void ClockStopperDemo()
        {
            bool done = false;
            _iterations = 0;

            _clock1.StopClock();

            Debug.Print("\n**********Doing Stop/Start Clock test**********");
            Debug.Print("The clock will stop incrementing for about 10 seconds then resume.");

            while (true)
            {
                if (_iterations > 20)
                {
                    _iterations = 0;
                    break;
                }

                var dt = _clock1.GetDateTime();
                Debug.Print("RTC DateTime - " + dt);

                if ((_iterations >= 10) && !done)
                {
                    done = true;
                    _clock1.StartClock();
                }

                Thread.Sleep(1000);
                _iterations += 1;
            }
        }

        private static void UserRamDemo()
        {
            Debug.Print("\n" + "**********Doing User Ram Test**********");

            // Test writing to arbitrary registers
            Debug.Print("\nWriting distinct RAM registers (writing the register number to itself)");
            for (int b = (byte)RTCClick.UserRam.StartAddressBlock; b <= (byte)RTCClick.UserRam.EndAddressBlock; b++)
            {
                _clock1.WriteUserRamAddress((byte)b, (byte)b);
            }

            // Test reading from arbitrary registers
            Debug.Print("Reading distinct RAM registers (the registers and values read should be the same)");
            for (int b = (byte)RTCClick.UserRam.StartAddressBlock; b <= (byte)RTCClick.UserRam.EndAddressBlock; b++)
            {
                Debug.Print(b + ": " + _clock1.ReadUserRamAddress((byte)b));
            }

            Debug.Print("\n");

            var ipsumLorem = "Lorem ipsum dolor sit amet, consectetur adipiscing elit. Integer nec odio. Praesent libero. Sed cursus ante dapibus diam. Sed nisi. Nulla quis sem at nibh elementum imperdiet. Duis sagittis ipsum. Praesent mauris. Fusce nec tellus.";
            byte[] userRam = new byte[(byte)RTCClick.UserRam.RamSize];
            Debug.Print("Writing a long string to User Ram - " + ipsumLorem);

            // Copy the string to the ram buffer
            for (byte b = 0; b <= ipsumLorem.Length - 1; b++)
            {
                userRam[b] = (byte)ipsumLorem[b];
            }
            // Write it to the User RAM.
            _clock1.WriteUserRam(userRam);

            Debug.Print("\n");

            //Test reading from the RAM as a single block
            Debug.Print("Reading from the RAM as a block of memory...");

            userRam = _clock1.ReadUserRam();

            var ramString = string.Empty;

            for (int i = 0; i <= userRam.Length - 1; i++)
            {
                ramString += (char)userRam[i];
            }

            if (ramString.Length != 0) Debug.Print("Reading from USER RAM: " + ramString + "\n");

            Debug.Print("Writing a char to a specific Ram address");
            _clock1.WriteUserRamAddress(0xFf, (byte)'c');
            Debug.Print("Reading back from the same address - " + (char)_clock1.ReadUserRamAddress(0xFF));
        }

        static void _clock1_AlarmTriggered(object sender, RTCClick.AlarmEventArgs e)
        {
            Debug.Print("Alarm of type - " + (e.AlarmType == RTCClick.AlarmBehavior.DailyAlarm ? "Daily Alarm" : e.AlarmType == RTCClick.AlarmBehavior.WeekDayAlarm ? "Weekday Alarm" : "Dated Alarm") + " - occurred at " + e.AlarmEventEventTime + "\n");
            //For Testing only so it will re-fire.
            _clock1.SetClock(new DateTime(2014, 8, 19, 23, 59, 55));
        }

    }
}